

local moduleName = "Basics"
local core = SimpleUnitFrames

local Basics = core:NewModule(moduleName, "AceEvent-2.0")

local styles = {

	["currmaxhp"] = function(unit) 
		return UnitHealth(unit) .. '/' .. UnitHealthMax(unit) 
	end,
	
	["percenthp"] = function(unit) 
		return string.format( "%d%%", UnitHealth(unit) * 100 / UnitHealthMax(unit) ) 
	end,
	
	["completehp"] = function(unit) 
		local curr, max = UnitHealth(unit), UnitHealthMax(unit)
		local percent = curr*100/max
		if max == 100 then -- Only return % when the exact HP is not know.
			return string.format( "(%d%%)", percent )
		else
			return string.format( "%d/%d (%d%%)", curr, max, percent)
		end
	end,
	
	["deficithp"] = function(unit)
		return UnitHealth(unit) - UnitHealthMax(unit)
	end,
	
	["mh3currmaxhp"] = function(unit)
		local curr, max
		if MobHealth3 then
			curr, max = MobHealth3:GetUnitHealth(unit)
		else
			curr, max = UnitHealth(unit), UnitHealthMax(unit)
		end
		return curr .. '/' .. max
	end,
	
	["mh3completehp"] = function(unit)
		local curr, max
		if MobHealth3 then
			curr, max = MobHealth3:GetUnitHealth(unit)
		else
			curr, max = UnitHealth(unit), UnitHealthMax(unit)
		end
		local percent = curr*100/max
		if max > 100000 then
			max = string.format("%dK", max/1000)
		end
		if curr > 100000 then
			curr = string.format("%dK", curr/1000)
		end
		if max == 100 then -- Only return % when the exact HP is not know.
			return string.format( "(%d%%)", percent )
		else
			return string.format( "%s/%s (%d%%)", curr, max, percent)
		end
	end,

	["currmaxmp"] = function(unit) 
		return UnitMana(unit) .. '/' .. UnitManaMax(unit) 
	end,
	
	["percentmp"] = function(unit) 
		return string.format( "%d%%", UnitMana(unit) * 100 / UnitManaMax(unit) ) 
	end,
	
	["completemp"] = function(unit) 
		local curr, max = UnitMana(unit), UnitManaMax(unit)
		return string.format( "%d/%d (%d%%)", curr, max, curr*100/max)
	end,

	["deficitmp"] = function(unit)
		return UnitMana(unit) - UnitManaMax(unit)
	end,
	
	["none"] = function() return "" end,
	
}

local unitMap = {}

local icons = {
	BEAST = true,
	CRITTER = true,
	DEMON = true,
	DRAGONKIN = true,
	DRUID = true,
	ELEMENTAL = true,
	GIANT = true,
	HUMANOID = true,
	HUNTER = true,
	MAGE = true,
	MECHANICAL = true,
	PALADIN = true,
	PRIEST = true,
	ROGUE = true,
	SHAMAN = true,
	UNDEAD = true,
	UNKNOWN = true,
	WARLOCK = true,
	WARRIOR = true,
}

local frames = {}

Basics.formatList = {
	"currmaxhp", "percenthp", "completehp", "deficithp", "mh3completehp", "mh3currmaxhp",
	"currmaxmp", "percentmp", "completemp", "deficitmp",
	"none",
}

Basics.unitMap = unitMap

Basics.consoleOptions = {}
Basics.defaultOptions = {} 
Basics.frameSettings = {}


function Basics:OnInitialize()
	self.db = core:AcquireDBNamespace(moduleName)

	core:RegisterDefaults(moduleName, 'profile', self.defaultOptions)
	
	local args = self.consoleOptions
	
	local settings = self.frameSettings

	for k, v in pairs(args) do
		core.consoleOptions.args[k] = v
	end
			

	self.frames = frames
	for unit, unitSettings in pairs(settings) do
		local frame = CreateFrame("Frame", nil, unitSettings.parent)
		frame:SetFrameLevel(unitSettings.parent:GetFrameLevel()+2)
		frame.fontStrings = {}
		frames[unit] = frame
		for text, textSettings in pairs(unitSettings.texts) do
			local fontString = frame:CreateFontString(nil, "OVERLAY", textSettings.font)
			fontString:SetPoint(unpack(textSettings.point))
			frame.fontStrings[text] = fontString
		end
		if unitSettings.classIcon then
			local texture = frame:CreateTexture("OVERLAY")
			texture:SetHeight(unitSettings.classIcon.size)
			texture:SetWidth(unitSettings.classIcon.size)
			texture:SetPoint( unpack(unitSettings.classIcon.point ) )
			frame.classIcon = texture
		end
	end
	
	self.consoleOptions = nil
	self.defaultOptions = nil
	self.frameSettings = nil
	
end

function Basics:OnEnable()

	self:RegisterEvent("UNIT_HEALTH", "Refresh")
	self:RegisterEvent("UNIT_MAXHEALTH", "Refresh")
	self:RegisterEvent("UNIT_MANA", "Refresh")
	self:RegisterEvent("UNIT_MAXMANA", "Refresh")
	self:RegisterEvent("UNIT_RAGE", "Refresh")
	self:RegisterEvent("UNIT_MAXRAGE", "Refresh")
	self:RegisterEvent("UNIT_ENERGY", "Refresh")
	self:RegisterEvent("UNIT_MAXENERGY", "Refresh")
	self:RegisterEvent("UNIT_DISPLAYPOWER", "Refresh")
	

	self:Refresh()
end

function Basics:OnDisable()
	self:UnregisterAllEvents()
	for unit, unitFrame in pairs(frames) do
		if unitFrame.classIcon then
			unitFrame.classIcon:Hide()
		end
		if unitFrame.fontStrings then
			for text, fontString in pairs(unitFrame.fontStrings) do
				fontString:SetText("")
			end
		end
	end
end

function Basics:Refresh(unit)
	if unit then
		if UnitExists(unit) then
			local mappedUnit = unitMap[unit] or unit
			if mappedUnit and frames[unit] then
				if frames[unit].classIcon then
					self:UpdateClassIcon(unit)
				end
				for text, fontString in pairs(frames[unit].fontStrings) do
					self:UpdateValue(unit, self.db.profile[mappedUnit][text], fontString )
				end
			end
		end
	else
		for unit in pairs(frames) do
			self:Refresh(unit)
		end
	end
end

function Basics:UpdateValue(unit, style, fontString)
	fontString:SetText( self:GetValueText(unit, style) )
end

function Basics:UpdateClassIcon(unit)
	
	local classIcon = frames[unit] and frames[unit].classIcon
	local mappedUnit = unitMap[unit] or unit
	
	if classIcon and mappedUnit and UnitExists(unit) then
	
		if self.db.profile[mappedUnit].class then
		
			local icon = nil
			if UnitIsPlayer(unit) then
				_, icon = UnitClass(unit)
			else
				icon = UnitCreatureType(unit)
				icon = icon and icon:upper()
			end
			
			if not icon or not icons[icon] then
				icon = "UNKNOWN"
			end
			
			classIcon:SetTexture("Interface\\AddOns\\SimpleUnitFrames\\Icons\\" .. icon .. ".tga")
			classIcon:Show()
		
		else
			
			classIcon:Hide()
			
		end
	end
	
end


function Basics:GetValueText(unit, style)
	if not styles[style] then return "ERROR" end
	return styles[style](unit)
end



